;  The WOZ Monitor for the Apple 1
;  Written by Steve Wozniak in 1976


; Page 0 Variables

XAML            = $24           ;  Last "opened" location Low
XAMH            = $25           ;  Last "opened" location High
STL             = $26           ;  Store address Low
STH             = $27           ;  Store address High
L               = $28           ;  Hex value parsing Low
H               = $29           ;  Hex value parsing High
YSAV            = $2A           ;  Used to see if hex value is given
MODE            = $2B           ;  $00=XAM, $7F=STOR, $AE=BLOCK XAM

KBD         = $A000

.macro KEYPRESS
WAITDW:     BIT KBD
            BPL WAITDW
WAITUP:     BIT KBD
            BMI WAITUP
.endmacro

; Other Variables

IN              = $0200         ;  Input buffer to $027F

            .export RESET
            .import DSP_INIT
            .import DSP_PRBYTE
            .import DSP_PUTCH
            .import DSP_NEWLINE
            .import DSP_CLS
            .import KBD_GETCH

;               .org $FF00
               .export RESET

RESET:      CLD             ; Clear decimal arithmetic mode.
            CLI
            JSR DSP_INIT
            LDA #$00
            TAY
NOTCR:      CMP #$7F        ; Backspace?
            BEQ BACKSPACE   ; Yes.
            CMP #$00        ; ESC?
            BEQ ESCAPE      ; Yes.
            JSR DSP_PUTCH   ; Display character.
            INY             ; Advance text index.
            CPY #17
            BMI NEXTCHAR    ; Auto ESC if > 16
ESCAPE:     JSR DSP_CLS     ; Clear display
            LDA #$2F        ; '/'
            JSR DSP_PUTCH   ; Output it.
GETLINE:    JSR DSP_NEWLINE ; CR.
            LDY #$01        ; Initialize text index.
BACKSPACE:  DEY             ; Back up text index.
            BMI ESCAPE      ; Beyond start of line, reinitialize.
NEXTCHAR:   JSR KBD_GETCH   ; Key ready?
            STA IN,Y        ; Add to text buffer.
            CMP #$0D        ; CR?
            BNE NOTCR       ; No.
            JSR DSP_CLS     ; Clear screen
            LDY #$FF        ; Reset text index.
            LDA #$00        ; For XAM mode.
            TAX             ; 0->X.
SETMODE:    ASL
            ASL             ; Leaves 11xxxxxx if setting STOR mode.
            STA MODE        ; $00=XAM 11xxxxxx=STOR 10xxxxxx=BLOCK XAM
BLSKIP:     INY             ; Advance text index.
NEXTITEM:   LDA IN,Y        ; Get character.
            CMP #$0D        ; CR?
            BEQ GETLINE     ; Yes, done this line.
            CMP #'.'        ; "."? ($2E)
            BCC BLSKIP      ; Skip delimiter. (char < '.')
            BEQ SETMODE     ; Yes. Set BLOCK XAM mode.
            CMP #':'        ; ":"? ($3A)
            BEQ SETMODE     ; Yes. Set STOR mode.
            CMP #'R'        ; "R"?
            BEQ RUN         ; Yes. Run user program.
            STX L           ; $00-> L.
            STX H           ; and H.
            STY YSAV        ; Save Y for comparison.
NEXTHEX:    LDA IN,Y        ; Get character for hex test.
            CMP #'0'
            BCC NOTHEX
            CMP #'9'+1
            BCC DIG
            CMP #'A'
            BCC NOTHEX
            CMP #'G'
            BCS NOTHEX
            ADC #256 + $0A - 'A' ; Potrei fare SBC, ma anche no... Carry is cleared
DIG:        AND #$0F
            ASL
            ASL             ; Hex digit to MSD of A.
            ASL
            ASL
            LDX #$04        ; Shift count.
HEXSHIFT:   ASL             ; Hex digit left, MSB to carry.
            ROL L           ; Rotate into LSD.
            ROL H           ;  Rotate into MSD’s.
            DEX             ; Done 4 shifts?
            BNE HEXSHIFT    ; No, loop.
            INY             ; Advance text index.
            BNE NEXTHEX     ; Always taken. Check next char for hex.
NOTHEX:     CPY YSAV        ; Check if L, H empty (no hex digits).
            BEQ ESCAPE      ; Yes, generate ESC sequence.
            BIT MODE        ; Test MODE byte.
            BVC NOTSTOR     ; B6=1 STOR 0 for XAM & BLOCK XAM
            LDA L           ; LSD’s of hex data.
            STA (STL,X)     ; Store at current ‘store index’.
            INC STL         ; Increment store index.
            BNE NEXTITEM    ; Get next item. (no carry).
            INC STH         ; Add carry to ‘store index’ high order.
TONEXTITEM: JMP NEXTITEM    ; Get next command item.
RUN:        JMP (XAML)      ; Run at current XAM index.
NOTSTOR:    BMI XAMNEXT     ; B7=0 for XAM, 1 for BLOCK XAM.
            LDX #$02        ; Byte count.
SETADR:     LDA L-1,X       ; Copy hex data to
            STA STL-1,X     ; ‘store index’.
            STA XAML-1,X    ; And to ‘XAM index’.
            DEX             ; Next of 2 bytes.
            BNE SETADR      ; Loop unless X=0.
NXTPRNT:    BNE PRDATA      ; NE means no address to print.
            KEYPRESS        ; Wait for user before printing next line
            JSR DSP_CLS     ; Make room for another line
            LDA XAMH        ; ‘Examine index’ high-order byte.
            JSR DSP_PRBYTE  ; Output it in hex format.
            LDA XAML        ; Low-order ‘examine index’ byte.
            JSR DSP_PRBYTE  ; Output it in hex format.
            LDA #':'        ; ":".
            JSR DSP_PUTCH   ; Output it.
PRDATA:     LDA (XAML,X)    ; Get data byte at ‘examine index’.
            JSR DSP_PRBYTE  ; Output it in hex format.
            LDA #' '        ; Blank.
            JSR DSP_PUTCH   ; Output it.
XAMNEXT:    STX MODE        ; 0->MODE (XAM mode).
            LDA XAML
            CMP L           ; Compare ‘examine index’ to hex data.
            LDA XAMH
            SBC H
            BCS TONEXTITEM  ; Not less, so no more data to output.
            INC XAML
            BNE MOD8CHK     ; Increment ‘examine index’.
            INC XAMH
MOD8CHK:    LDA XAML        ; Check low-order ‘examine index’ byte
            AND #$03        ; For MOD 4=0
            BPL NXTPRNT     ; Always taken.

; Interrupt Vectors

;                .WORD $0F00     ; NMI
;                .WORD RESET     ; RESET
;                .WORD $0000     ; BRK/IRQ
