## Directories ##
## This is where the source files are located,
## which are not in the current directory
SRC_DIR      ?= .

INC_DIR      ?= $(SRC_DIR)

BUILD_DIR    ?= build
OUTPUT_DIR   ?= output

######################################################################
#                                                                    #
######################################################################

## Object files
## Automatically declares object file names
OBJS         := $(patsubst %.c,   $(BUILD_DIR)/%.o,  $(filter %.c,$(SRCS)) )
OBJS         += $(patsubst %.cpp, $(BUILD_DIR)/%.o,  $(filter %.cpp,$(SRCS)) )
OBJS         += $(patsubst %.s,   $(BUILD_DIR)/%.o,  $(filter %.s,$(SRCS)) )
OBJS         += $(patsubst %.S,   $(BUILD_DIR)/%.o,  $(filter %.S,$(SRCS)) )
OBJS         += $(patsubst %.asm, $(BUILD_DIR)/%.o,  $(filter %.asm,$(SRCS)) )

## Virtual Paths
## Tell make to look in that folder if it cannot find a source
## from the current directory
vpath %.c   $(SRC_DIR)
vpath %.cpp $(SRC_DIR)
vpath %.S   $(SRC_DIR)
vpath %.s   $(SRC_DIR)
vpath %.asm $(SRC_DIR)
vpath %.h   $(INC_DIR)

######################################################################
#                         SETUP TOOLS                                #
######################################################################
ECHO      := /bin/echo

## GCC/programming Tools
# ifdef GCC_PATH
CC      = cc65
# CXX     = $(GCC_PATH)/$(PREFIX)g++
# OBJCOPY = $(GCC_PATH)/$(PREFIX)objcopy
# OBJDUMP = $(GCC_PATH)/$(PREFIX)objdump
# GDB     = $(GCC_PATH)/$(PREFIX)gdb
AS      = ca65
LD      = ld65
# SIZE    = $(GCC_PATH)/$(PREFIX)size
# else
# CC      = $(PREFIX)gcc
# CXX     = $(PREFIX)g++
# OBJCOPY = $(PREFIX)objcopy
# OBJDUMP = $(PREFIX)objdump
# GDB     = $(PREFIX)gdb
# AS      = $(PREFIX)as
# SIZE    = $(PREFIX)size
# endif

######################################################################
#                      PROGRAMMING TOOLS                             #
######################################################################
## With ST-Link
PRG      = ~/Workspace/Retrofficina/Programmer/sw/serprog
PRGFLAGS   = -a 0 -w
## With stm32flash
# PRG        = stm32flash
# PRGPORT   ?= /dev/ttyUSB0
# PRGFLAGS   = $(PRGPORT)
# PRGFLAGS  += -w # mind the whitespace

## With avrdude
# PRG        = avrdude
# PRGPORT   ?= /dev/ttyUSB0
# PRGFLAGS  += -c arduino -P $(PRGPORT) -p $(MCU) -U flash:w:

######################################################################
#                             TARGETS                                #
######################################################################
all:     $(OUTPUT_DIR)/$(PROJ_NAME).hex

## Avoid generating deps if cleaning
## https://codereview.stackexchange.com/questions/2547/makefile-dependency-generation
ifneq ($(MAKECMDGOALS),clean)
-include $(DEPS)
endif
# $(OUTPUT_DIR)/$(PROJ_NAME).elf: $(OBJS)

## asm ##
$(BUILD_DIR)/%.o : %.S
	@echo -e "\033[1;33m[Assembling  ]\033[0m AS $<"
	@mkdir -p `dirname $@`
	$(AS) $(CFLAGS) $< -o $@

## asm ##
$(BUILD_DIR)/%.o : %.asm
	@echo -e "\033[1;33m[Assembling  ]\033[0m AS $<"
	@mkdir -p `dirname $@`
	$(AS) $(CFLAGS) $< -o $@

## asm ##
$(BUILD_DIR)/%.o : %.s
	@echo -e "\033[1;33m[Assembling  ]\033[0m AS $<"
	@mkdir -p `dirname $@`
	$(AS) $(CFLAGS) $< -o $@

## .cxx files ##
# $(BUILD_DIR)/%.o:  %.cpp
# 	@echo -e "\033[1;33m[Compiling   ]\033[0m CX $<"
# 	@mkdir -p `dirname $@`
# 	$(CXX) $(CXXFLAGS) -c $< -o $@

## .c files ##
# $(BUILD_DIR)/%.o:  %.c
# 	@echo -e "\033[1;33m[Compiling   ]\033[0m CC $<"
# 	@mkdir -p `dirname $@`
# 	$(CC) $(CFLAGS) -c $< -o $@

# .DELETE_ON_ERROR:
# $(BUILD_DIR)/%.d: %.c
# 	@mkdir -p `dirname $@`
# 	@$(ECHO) -n "$@ " > $@
# 	@$(CC) -MM -MP -MT '$(@D)/$(basename $(<F)).o' $(CFLAGS) $< >> $@

# .DELETE_ON_ERROR:
# $(BUILD_DIR)/%.d: %.cpp
# 	@mkdir -p `dirname $@`
# 	@$(ECHO) -n "$@ " > $@
# 	@$(CC) -MM -MP -MT '$(@D)/$(basename $(<F)).o' $(CFLAGS) $< >> $@

## Linking ##
$(OUTPUT_DIR)/$(PROJ_NAME).hex: $(OBJS)
	@echo -e "\033[1;34m[Linking     ]\033[0m $@"
	@mkdir -p ${OUTPUT_DIR}
	$(LD) $(LDFLAGS) -o $@ $^
#	 @echo -e "\033[1;35m[Disasm...   ]\033[0m $^"
#	 $(OBJDUMP) -S $@ > $(OUTPUT_DIR)/$(PROJ_NAME).lss

## Extracting ##
# $(OUTPUT_DIR)/$(PROJ_NAME).hex: $(OUTPUT_DIR)/$(PROJ_NAME).elf
# 	@echo -e "\033[1;36m[Binary      ]\033[0m $^"
# 	$(OBJCOPY) -O ihex -R .eeprom $< $@

.PHONY: flash
flash: $(OUTPUT_DIR)/$(PROJ_NAME).hex
	@echo -e "\033[1;36m[Flash      ]\033[0m $^"
	$(PRG) $(PRGFLAGS) $<

.PHONY: size
size: $(OUTPUT_DIR)/$(PROJ_NAME).elf
	$(SIZE) $(OUTPUT_DIR)/$(PROJ_NAME).elf

.PHONY: clean
clean:
	@echo -e "\033[1;33m[Cleaning   ]\033[0m"
	@rm -rf $(BUILD_DIR)/*
	@rm -rf $(OUTPUT_DIR)/*